// dibview.cpp : implementation of the CDibView class
//
// This is a part of the Microsoft Foundation Classes C++ library.
// Copyright (C) 1992-1997 Microsoft Corporation
// All rights reserved.
//
// This source code is only intended as a supplement to the
// Microsoft Foundation Classes Reference and related
// electronic documentation provided with the library.
// See these sources for detailed information regarding the
// Microsoft Foundation Classes product.

#include "stdafx.h"
#include "diblook.h"

#include "dibdoc.h"
#include "dibview.h"
#include "dibapi.h"
#include "mainfrm.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDibView

IMPLEMENT_DYNCREATE(CDibView, CScrollView)

BEGIN_MESSAGE_MAP(CDibView, CScrollView)
	//{{AFX_MSG_MAP(CDibView)
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCopy)
	ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
	ON_COMMAND(ID_INDICATOR_POS, OnIndicatorPos)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_POS, OnUpdateIndicatorPos)
	ON_MESSAGE(WM_DOREALIZE, OnDoRealize)
	ON_COMMAND(ID_POPUP1_MOVEFRAME, OnPopup1MoveFrame)
	ON_COMMAND(ID_POPUP1_DEFINEFRAME, OnPopup1DefineFrame)
	ON_COMMAND(ID_POPUP1_SAVEFRAMEAS, OnPopup1SaveFrameAs)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_RBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	//}}AFX_MSG_MAP

	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CScrollView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDibView construction/destruction

CDibView::CDibView()
{
	m_left = 0;
	m_top = 0;
	m_right = 0;
	m_bottom = 0;
	m_flag = 0;
	m_x = 0;
	m_y = 0;

	if (((CDibLookApp *)AfxGetApp())->m_define_frame)
	{
		m_right -= ((CDibLookApp *)AfxGetApp())->m_width;
		m_bottom -= ((CDibLookApp *)AfxGetApp())->m_height;
	}
}

CDibView::~CDibView()
{
}

/////////////////////////////////////////////////////////////////////////////
// CDibView drawing

void CDibView::OnDraw(CDC* pDC)
{
	CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();

	CDibDoc* pDoc = GetDocument();

	HDIB hDIB = pDoc->GetHDIB();
	if (hDIB == NULL)
	{
		CRect rcDisp;
		GetClientRect(rcDisp);
		rcDisp += GetDeviceScrollPosition();
		pDC->FillSolidRect(rcDisp, 0x00808080L);
		return;
	}

	LPSTR lpDIB = (LPSTR) ::GlobalLock((HGLOBAL) hDIB);
	int cxDIB = (int) ::DIBWidth(lpDIB);         // Size of DIB - x
	int cyDIB = (int) ::DIBHeight(lpDIB);        // Size of DIB - y
	::GlobalUnlock((HGLOBAL) hDIB);
	CRect rcDIB;
	rcDIB.top = rcDIB.left = 0;
	rcDIB.right = cxDIB;
	rcDIB.bottom = cyDIB;
	CRect rcDest;
	if (pDC->IsPrinting())   // printer DC
	{
		// get size of printer page (in pixels)
		int cxPage = pDC->GetDeviceCaps(HORZRES);
		int cyPage = pDC->GetDeviceCaps(VERTRES);
		// get printer pixels per inch
		int cxInch = pDC->GetDeviceCaps(LOGPIXELSX);
		int cyInch = pDC->GetDeviceCaps(LOGPIXELSY);

		//
		// Best Fit case -- create a rectangle which preserves
		// the DIB's aspect ratio, and fills the page horizontally.
		//
		// The formula in the "->bottom" field below calculates the Y
		// position of the printed bitmap, based on the size of the
		// bitmap, the width of the page, and the relative size of
		// a printed pixel (cyInch / cxInch).
		//
		rcDest.top = rcDest.left = 0;
		rcDest.bottom = (int)(((double)cyDIB * cxPage * cyInch)
				/ ((double)cxDIB * cxInch));
		rcDest.right = cxPage;
	}
	else   // not printer DC
	{
		rcDest = rcDIB;
		CRect rcDisp;
		GetClientRect(rcDisp);
		rcDisp += GetDeviceScrollPosition();
		if (rcDisp.right > rcDest.right &&
			rcDisp.bottom > rcDest.bottom)
		{
			rcDisp.left = rcDest.right;
			rcDisp.bottom = rcDest.bottom;
			pDC->FillSolidRect(rcDisp, 0x00808080L);
			GetClientRect(rcDisp);
			rcDisp += GetDeviceScrollPosition();
			rcDisp.top = rcDest.bottom;
			pDC->FillSolidRect(rcDisp, 0x00808080L);
		}
		else if (rcDisp.right > rcDest.right)
		{
			rcDisp.left = rcDest.right;
			pDC->FillSolidRect(rcDisp, 0x00808080L);
		}
		else if (rcDisp.bottom > rcDest.bottom)
		{
			rcDisp.top = rcDest.bottom;
			pDC->FillSolidRect(rcDisp, 0x00808080L);
		}
	}
	::PaintDIB(pDC->m_hDC, &rcDest, pDoc->GetHDIB(),
		&rcDIB, pDoc->GetDocPalette());

	if (pDC->IsPrinting()) return;

	if (myapp->m_show_blockgrid)
	{
		CRect rect;
		GetClientRect(rect);
		rect += GetDeviceScrollPosition();
		if (rect.right > rcDIB.right) rect.right = rcDIB.right;
		if (rect.bottom > rcDIB.bottom) rect.bottom = rcDIB.bottom;
	#if 0
		LOGBRUSH mybrush;
		mybrush.lbStyle = BS_SOLID;
		mybrush.lbColor = (COLORREF)0x00000000L;
		DWORD mystyle[2];
		mystyle[0] = 1;
		mystyle[1] = 1;
		CPen mypen(PS_USERSTYLE | PS_COSMETIC, 1, &mybrush, 2, mystyle);
	#else
		// CPen mypen(PS_ALTERNATE | PS_COSMETIC, 1, (COLORREF)0x00000000L);
		CPen mypen(0, 0, (COLORREF)0x00555555L);
	#endif
		CPen *pold = pDC->SelectObject(&mypen);
		// pDC->SetBkMode(TRANSPARENT);

		int i = 0; do i += pDoc->m_MCUwidth; while (i < rect.left);
		for (; i < rect.right; i += pDoc->m_MCUwidth)
		{
		  pDC->MoveTo(i, 0);
		  pDC->LineTo(i, rect.bottom);
		}

		i = 0; do i += pDoc->m_MCUheight; while (i < rect.top);
		for (; i < rect.bottom; i += pDoc->m_MCUheight)
		{
		  pDC->MoveTo(0, i);
		  pDC->LineTo(rect.right, i);
		}

		pDC->SelectObject(pold);
	}

	if (m_right < 0) if ((m_right += cxDIB) < 0) m_right = 0;
	if (m_bottom < 0) if ((m_bottom += cyDIB) < 0) m_bottom = 0;

	pDoc->m_crop_xoffset = m_left;
	pDoc->m_crop_yoffset = m_top;
	pDoc->m_crop_width = cxDIB - m_right - m_left;
	pDoc->m_crop_height = cyDIB - m_bottom - m_top;

	if (m_left || m_top || m_right || m_bottom)
	{
		CPen mypen(PS_DOT, 0, (COLORREF)0x00000000L);
		CPen *pold = pDC->SelectObject(&mypen);
		pDC->SetBkColor((COLORREF)0x00FFFFFFL);
		int x1, y1, x2, y2;
		x1 = m_left;
		y1 = m_top;
		x2 = cxDIB - 1 - m_right;
		y2 = cyDIB - 1 - m_bottom;
		pDC->MoveTo(x1, y1);
		pDC->LineTo(x2, y1);
		pDC->LineTo(x2, y2);
		pDC->LineTo(x1, y2);
		pDC->LineTo(x1, y1);
		pDC->SelectObject(pold);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CDibView printing

BOOL CDibView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

/////////////////////////////////////////////////////////////////////////////
// CDibView commands


LRESULT CDibView::OnDoRealize(WPARAM wParam, LPARAM)
{
	ASSERT(wParam != NULL);
	CDibDoc* pDoc = GetDocument();
	if (pDoc->GetHDIB() == NULL)
		return 0L;  // must be a new document

	CPalette* pPal = pDoc->GetDocPalette();
	if (pPal != NULL)
	{
		CMainFrame* pAppFrame = (CMainFrame*) AfxGetApp()->m_pMainWnd;
		ASSERT_KINDOF(CMainFrame, pAppFrame);

		CClientDC appDC(pAppFrame);
		// All views but one should be a background palette.
		// wParam contains a handle to the active view, so the SelectPalette
		// bForceBackground flag is FALSE only if wParam == m_hWnd (this view)
		CPalette* oldPalette = appDC.SelectPalette(pPal, ((HWND)wParam) != m_hWnd);

		if (oldPalette != NULL)
		{
			UINT nColorsChanged = appDC.RealizePalette();
			if (nColorsChanged > 0)
				pDoc->UpdateAllViews(NULL);
			appDC.SelectPalette(oldPalette, TRUE);
		}
		else
		{
			TRACE0("\tSelectPalette failed in CDibView::OnPaletteChanged\n");
		}
	}
	SetClassLong(m_hWnd, GCL_HBRBACKGROUND, NULL);

	return 0L;
}

void CDibView::OnInitialUpdate()
{
	CScrollView::OnInitialUpdate();
	ASSERT(GetDocument() != NULL);

	SetScrollSizes(MM_TEXT, GetDocument()->GetDocSize());
	SetClassLong(m_hWnd, GCL_HBRBACKGROUND, NULL);
}


void CDibView::OnActivateView(BOOL bActivate, CView* pActivateView,
					CView* pDeactiveView)
{
	CScrollView::OnActivateView(bActivate, pActivateView, pDeactiveView);

	if (bActivate)
	{
		ASSERT(pActivateView == this);
		OnDoRealize((WPARAM)m_hWnd, 0);   // same as SendMessage(WM_DOREALIZE);

		HDIB hDIB = GetDocument()->GetHDIB();
		if (hDIB == NULL) return;
		LPSTR lpDIB = (LPSTR) ::GlobalLock((HGLOBAL) hDIB);
		int cxDIB = (int) ::DIBWidth(lpDIB);         // Size of DIB - x
		int cyDIB = (int) ::DIBHeight(lpDIB);        // Size of DIB - y
		::GlobalUnlock((HGLOBAL) hDIB);

		if (m_right < 0) if ((m_right += cxDIB) < 0) m_right = 0;
		if (m_bottom < 0) if ((m_bottom += cyDIB) < 0) m_bottom = 0;

		char buf[64];
		unsigned int paru[2];
		int pari[2];
		sprintf(buf, " %dx%d+%d+%d",
			cxDIB - m_right - m_left,
			cyDIB - m_bottom - m_top, m_left, m_top);
		((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
		((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
		((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);
	}
	SetClassLong(m_hWnd, GCL_HBRBACKGROUND, NULL);
}

void CDibView::OnEditCopy()
{
	CDibDoc* pDoc = GetDocument();
	// Clean clipboard of contents, and copy the DIB.

	if (OpenClipboard())
	{
		BeginWaitCursor();
		EmptyClipboard();
		SetClipboardData (CF_DIB, CopyHandle((HANDLE) pDoc->GetHDIB()) );
		CloseClipboard();
		EndWaitCursor();
	}
}



void CDibView::OnUpdateEditCopy(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(GetDocument()->GetHDIB() != NULL);
}


void CDibView::OnEditPaste()
{
	HDIB hNewDIB = NULL;

	if (OpenClipboard())
	{
		BeginWaitCursor();

		hNewDIB = (HDIB) CopyHandle(::GetClipboardData(CF_DIB));

		CloseClipboard();

		if (hNewDIB != NULL)
		{
			CDibDoc* pDoc = GetDocument();
			pDoc->ReplaceHDIB(hNewDIB); // and free the old DIB
			pDoc->InitDIBData();    // set up new size & palette
			pDoc->SetModifiedFlag(TRUE);

			SetScrollSizes(MM_TEXT, pDoc->GetDocSize());
			OnDoRealize((WPARAM)m_hWnd,0);  // realize the new palette
			pDoc->UpdateAllViews(NULL);
		}
		EndWaitCursor();
	}
}


void CDibView::OnUpdateEditPaste(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(::IsClipboardFormatAvailable(CF_DIB));
}


void CDibView::OnIndicatorPos()
{
}


void CDibView::OnUpdateIndicatorPos(CCmdUI* pCmdUI)
{
}


void CDibView::OnLButtonDown(UINT nFlags, CPoint point)
{
	CScrollView::OnLButtonDown(nFlags, point);

	if (m_flag == 0)
	{
		m_x = point.x;
		m_y = point.y;
		m_flag = 1;
	}
}


void CDibView::OnLButtonUp(UINT nFlags, CPoint point)
{
	CScrollView::OnLButtonUp(nFlags, point);

	if (m_flag)
	{
		if (m_flag == 2 || m_flag == 101)
			GetDocument()->UpdateAllViews(NULL);
		m_flag = 0;
	}
}


void CDibView::OnRButtonDown(UINT nFlags, CPoint point)
{
	CScrollView::OnRButtonDown(nFlags, point);

	CMenu mnuTop;
	mnuTop.LoadMenu(IDR_POPUP1);
	CMenu* pPopup = mnuTop.GetSubMenu(0);
	ClientToScreen(&point);
	pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_LEFTBUTTON,
		point.x, point.y, this);
}


void CDibView::OnMouseMove(UINT nFlags, CPoint point)
{
  CScrollView::OnMouseMove(nFlags, point);

  CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
  CDibDoc *pDoc = GetDocument();

  CPoint scroll = GetDeviceScrollPosition();
  int x = point.x + scroll.x;
  int y = point.y + scroll.y;
  char buf[64];
  unsigned int paru[2];
  int pari[2];

  HDIB hDIB = pDoc->GetHDIB();
  if (hDIB == NULL) return;
  LPSTR lpDIB = (LPSTR) ::GlobalLock((HGLOBAL) hDIB);
  int cxDIB = (int) ::DIBWidth(lpDIB);         // Size of DIB - x
  int cyDIB = (int) ::DIBHeight(lpDIB);        // Size of DIB - y
  ::GlobalUnlock((HGLOBAL) hDIB);

  if (m_flag == 100)
  {
	int width = cxDIB - m_right - m_left;
	int height = cyDIB - m_bottom - m_top;
	m_left = x - width / 2; m_left -= m_left % pDoc->m_MCUwidth;
	if (m_left < 0) m_left = 0;
	m_top = y - height / 2; m_top -= m_top % pDoc->m_MCUheight;
	if (m_top < 0) m_top = 0;
	m_right = cxDIB - m_left - width;
	if (m_right < 0)
	{
	  m_left += m_right; m_right = m_left % pDoc->m_MCUwidth;
	  m_left -= m_right;
	}
	m_bottom = cyDIB - m_top - height;
	if (m_bottom < 0)
	{
	  m_top += m_bottom; m_bottom = m_top % pDoc->m_MCUheight;
	  m_top -= m_bottom;
	}
	SIZE size;
	RECT rect2;
	size.cx = 1;
	size.cy = 1;
	rect2.left = m_left - scroll.x;
	rect2.top = m_top - scroll.y;
	rect2.right = cxDIB - m_right - scroll.x;
	rect2.bottom = cyDIB - m_bottom - scroll.y;
	GetDC()->DrawDragRect(&rect2, size, NULL, size);
	m_flag = 101;
	return;
  }
  if (m_flag == 101)
  {
	SIZE size;
	RECT rect1, rect2;
	size.cx = 1;
	size.cy = 1;
	rect1.left = m_left - scroll.x;
	rect1.top = m_top - scroll.y;
	rect1.right = cxDIB - m_right - scroll.x;
	rect1.bottom = cyDIB - m_bottom - scroll.y;
	int width = cxDIB - m_right - m_left;
	int height = cyDIB - m_bottom - m_top;
	m_left = x - width / 2; m_left -= m_left % pDoc->m_MCUwidth;
	if (m_left < 0) m_left = 0;
	m_top = y - height / 2; m_top -= m_top % pDoc->m_MCUheight;
	if (m_top < 0) m_top = 0;
	m_right = cxDIB - m_left - width;
	if (m_right < 0)
	{
	  m_left += m_right; m_right = m_left % pDoc->m_MCUwidth;
	  m_left -= m_right;
	}
	m_bottom = cyDIB - m_top - height;
	if (m_bottom < 0)
	{
	  m_top += m_bottom; m_bottom = m_top % pDoc->m_MCUheight;
	  m_top -= m_bottom;
	}
	rect2.left = m_left - scroll.x;
	rect2.top = m_top - scroll.y;
	rect2.right = cxDIB - m_right - scroll.x;
	rect2.bottom = cyDIB - m_bottom - scroll.y;
	GetDC()->DrawDragRect(&rect2, size, &rect1, size);

//	CRect rcDisp;
//	GetClientRect(rcDisp);
//	if (point.x < rcDisp.left)
//		ScrollToPosition(CPoint(point.x, scroll.y));

	sprintf(buf, " %dx%d+%d+%d",
		cxDIB - m_right - m_left,
		cyDIB - m_bottom - m_top, m_left, m_top);
	((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
	((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
	((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);
	return;
  }

  if (nFlags & MK_LBUTTON)
  {
	switch (m_flag) {
	case 1:
	  if (point.x == m_x && point.y == m_y) break;
	  m_left = m_x + scroll.x;
	  m_top = m_y + scroll.y;
	  if (m_left >= cxDIB) m_left = cxDIB - 1;
	  if (m_top >= cyDIB) m_top = cyDIB - 1;
	  m_left -= m_left % pDoc->m_MCUwidth;
	  m_top -= m_top % pDoc->m_MCUheight;
	  if (x < m_left) x = m_left;
	  if (y < m_top) y = m_top;
	  if (myapp->m_endpoint_snap)
	  {
		x -= x % pDoc->m_MCUwidth;
		x += pDoc->m_MCUwidth - 1;
		y -= y % pDoc->m_MCUheight;
		y += pDoc->m_MCUheight - 1;
	  }
	  if (x >= cxDIB) x = cxDIB - 1;
	  if (y >= cyDIB) y = cyDIB - 1;
	  SIZE size;
	  RECT rect2;
	  size.cx = 1;
	  size.cy = 1;
	  rect2.left = m_left - scroll.x;
	  rect2.top = m_top - scroll.y;
	  m_right = cxDIB - 1 - x;
	  m_bottom = cyDIB - 1 - y;
	  rect2.right = cxDIB - m_right - scroll.x;
	  rect2.bottom = cyDIB - m_bottom - scroll.y;
	  GetDC()->DrawDragRect(&rect2, size, NULL, size);
	  m_flag = 2;
	  break;
	case 2:
	  if (x < m_left) x = m_left;
	  if (y < m_top) y = m_top;
	  if (myapp->m_endpoint_snap)
	  {
		x -= x % pDoc->m_MCUwidth;
		x += pDoc->m_MCUwidth - 1;
		y -= y % pDoc->m_MCUheight;
		y += pDoc->m_MCUheight - 1;
	  }
	  if (x >= cxDIB) x = cxDIB - 1;
	  if (y >= cyDIB) y = cyDIB - 1;
	  RECT rect1;
	  size.cx = 1;
	  size.cy = 1;
	  rect1.left = m_left - scroll.x;
	  rect1.top = m_top - scroll.y;
	  rect1.right = cxDIB - m_right - scroll.x;
	  rect1.bottom = cyDIB - m_bottom - scroll.y;
	  rect2.left = m_left - scroll.x;
	  rect2.top = m_top - scroll.y;
	  m_right = cxDIB - 1 - x;
	  m_bottom = cyDIB - 1 - y;
	  rect2.right = cxDIB - m_right - scroll.x;
	  rect2.bottom = cyDIB - m_bottom - scroll.y;
	  GetDC()->DrawDragRect(&rect2, size, &rect1, size);

	  sprintf(buf, " %dx%d+%d+%d",
		cxDIB - m_right - m_left,
		cyDIB - m_bottom - m_top, m_left, m_top);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);
	  break;
	case 4:
	  if (x > cxDIB - 1 - m_right) x = cxDIB - 1 - m_right;
	  x -= x % pDoc->m_MCUwidth;
	  SetCursor(LoadCursor(NULL, IDC_SIZEWE));
	  if (x == m_left) break;
	  m_left = x;
	  sprintf(buf, " %dx%d+%d+%d",
		cxDIB - m_right - m_left,
		cyDIB - m_bottom - m_top, m_left, m_top);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);
	  GetDocument()->UpdateAllViews(NULL);
	  break;
	case 5:
	  if (y > cyDIB - 1 - m_bottom) y = cyDIB - 1 - m_bottom;
	  y -= y % pDoc->m_MCUheight;
	  SetCursor(LoadCursor(NULL, IDC_SIZENS));
	  if (y == m_top) break;
	  m_top = y;
	  sprintf(buf, " %dx%d+%d+%d",
		cxDIB - m_right - m_left,
		cyDIB - m_bottom - m_top, m_left, m_top);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);
	  GetDocument()->UpdateAllViews(NULL);
	  break;
	case 6:
	  if (x < m_left) x = m_left;
	  if (myapp->m_endpoint_snap)
	  {
		x -= x % pDoc->m_MCUwidth;
		x += pDoc->m_MCUwidth - 1;
	  }
	  if (x >= cxDIB) x = cxDIB - 1;
	  SetCursor(LoadCursor(NULL, IDC_SIZEWE));
	  if (cxDIB - 1 - x == m_right) break;
	  m_right = cxDIB - 1 - x;
	  sprintf(buf, " %dx%d+%d+%d",
		cxDIB - m_right - m_left,
		cyDIB - m_bottom - m_top, m_left, m_top);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);
	  GetDocument()->UpdateAllViews(NULL);
	  break;
	case 7:
	  if (y < m_top) y = m_top;
	  if (myapp->m_endpoint_snap)
	  {
		y -= y % pDoc->m_MCUheight;
		y += pDoc->m_MCUheight - 1;
	  }
	  if (y >= cyDIB) y = cyDIB - 1;
	  SetCursor(LoadCursor(NULL, IDC_SIZENS));
	  if (cyDIB - 1 - y == m_bottom) break;
	  m_bottom = cyDIB - 1 - y;
	  sprintf(buf, " %dx%d+%d+%d",
		cxDIB - m_right - m_left,
		cyDIB - m_bottom - m_top, m_left, m_top);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);
	  GetDocument()->UpdateAllViews(NULL);
	  break;
	case 8:
	  if (x > cxDIB - 1 - m_right) x = cxDIB - 1 - m_right;
	  if (y > cyDIB - 1 - m_bottom) y = cyDIB - 1 - m_bottom;
	  x -= x % pDoc->m_MCUwidth;
	  y -= y % pDoc->m_MCUheight;
	  SetCursor(LoadCursor(NULL, IDC_SIZENWSE));
	  if (x == m_left && y == m_top) break;
	  m_left = x;
	  m_top = y;
	  sprintf(buf, " %dx%d+%d+%d",
		cxDIB - m_right - m_left,
		cyDIB - m_bottom - m_top, m_left, m_top);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);
	  GetDocument()->UpdateAllViews(NULL);
	  break;
	case 9:
	  if (x < m_left) x = m_left;
	  if (myapp->m_endpoint_snap)
	  {
		x -= x % pDoc->m_MCUwidth;
		x += pDoc->m_MCUwidth - 1;
	  }
	  if (x >= cxDIB) x = cxDIB - 1;
	  if (y > cyDIB - 1 - m_bottom) y = cyDIB - 1 - m_bottom;
	  y -= y % pDoc->m_MCUheight;
	  SetCursor(LoadCursor(NULL, IDC_SIZENESW));
	  if (cxDIB - 1 - x == m_right && y == m_top) break;
	  m_right = cxDIB - 1 - x;
	  m_top = y;
	  sprintf(buf, " %dx%d+%d+%d",
		cxDIB - m_right - m_left,
		cyDIB - m_bottom - m_top, m_left, m_top);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);
	  GetDocument()->UpdateAllViews(NULL);
	  break;
	case 10:
	  if (x < m_left) x = m_left;
	  if (y < m_top) y = m_top;
	  if (myapp->m_endpoint_snap)
	  {
		x -= x % pDoc->m_MCUwidth;
		x += pDoc->m_MCUwidth - 1;
		y -= y % pDoc->m_MCUheight;
		y += pDoc->m_MCUheight - 1;
	  }
	  if (x >= cxDIB) x = cxDIB - 1;
	  if (y >= cyDIB) y = cyDIB - 1;
	  SetCursor(LoadCursor(NULL, IDC_SIZENWSE));
	  if (cxDIB - 1 - x == m_right && cyDIB - 1 - y == m_bottom) break;
	  m_right = cxDIB - 1 - x;
	  m_bottom = cyDIB - 1 - y;
	  sprintf(buf, " %dx%d+%d+%d",
		cxDIB - m_right - m_left,
		cyDIB - m_bottom - m_top, m_left, m_top);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);
	  GetDocument()->UpdateAllViews(NULL);
	  break;
	case 11:
	  if (x > cxDIB - 1 - m_right) x = cxDIB - 1 - m_right;
	  x -= x % pDoc->m_MCUwidth;
	  if (y < m_top) y = m_top;
	  if (myapp->m_endpoint_snap)
	  {
		y -= y % pDoc->m_MCUheight;
		y += pDoc->m_MCUheight - 1;
	  }
	  if (y >= cyDIB) y = cyDIB - 1;
	  SetCursor(LoadCursor(NULL, IDC_SIZENESW));
	  if (x == m_left && cyDIB - 1 - y == m_bottom) break;
	  m_left = x;
	  m_bottom = cyDIB - 1 - y;
	  sprintf(buf, " %dx%d+%d+%d",
		cxDIB - m_right - m_left,
		cyDIB - m_bottom - m_top, m_left, m_top);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);
	  GetDocument()->UpdateAllViews(NULL);
	  break;
	}
  }
  else
  {
	if (x - m_left >= -1 && x - m_left <= 1 &&
		y - m_top >= -1 && y - m_top <= 1)
	{
	  SetCursor(LoadCursor(NULL, IDC_SIZENWSE));
	  m_flag = 8;
	}
	else if (x - (cxDIB - 1 - m_right) >= -1 && x - (cxDIB - 1 - m_right) <= 1 &&
			 y - m_top >= -1 && y - m_top <= 1)
	{
	  SetCursor(LoadCursor(NULL, IDC_SIZENESW));
	  m_flag = 9;
	}
	else if (x - (cxDIB - 1 - m_right) >= -1 && x - (cxDIB - 1 - m_right) <= 1 &&
			 y - (cyDIB - 1 - m_bottom) >= -1 && y - (cyDIB - 1 - m_bottom) <= 1)
	{
	  SetCursor(LoadCursor(NULL, IDC_SIZENWSE));
	  m_flag = 10;
	}
	else if (x - m_left >= -1 && x - m_left <= 1 &&
			 y - (cyDIB - 1 - m_bottom) >= -1 && y - (cyDIB - 1 - m_bottom) <= 1)
	{
	  SetCursor(LoadCursor(NULL, IDC_SIZENESW));
	  m_flag = 11;
	}
	else if (x - m_left >= -1 && x - m_left <= 1)
	{
	  SetCursor(LoadCursor(NULL, IDC_SIZEWE));
	  m_flag = 4;
	}
	else if (y - m_top >= -1 && y - m_top <= 1)
	{
	  SetCursor(LoadCursor(NULL, IDC_SIZENS));
	  m_flag = 5;
	}
	else if (x - (cxDIB - 1 - m_right) >= -1 && x - (cxDIB - 1 - m_right) <= 1)
	{
	  SetCursor(LoadCursor(NULL, IDC_SIZEWE));
	  m_flag = 6;
	}
	else if (y - (cyDIB - 1 - m_bottom) >= -1 && y - (cyDIB - 1 - m_bottom) <= 1)
	{
	  SetCursor(LoadCursor(NULL, IDC_SIZENS));
	  m_flag = 7;
	}
	else if (m_flag)
	{
	  SetCursor(LoadCursor(NULL, IDC_ARROW));
	  m_flag = 0;
	}
  }
}


void CDibView::OnPopup1MoveFrame()
{
  m_flag = 100;
}


class CFrameDlg : public CDialog
{
public:
	CFrameDlg() : CDialog(CFrameDlg::IDD)
		{
			//{{AFX_DATA_INIT(CFrameDlg)
			//}}AFX_DATA_INIT
		}
// Dialog Data
	//{{AFX_DATA(CFrameDlg)
		enum { IDD = IDD_FRAMEBOX };
	//}}AFX_DATA
// Implementation
protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	virtual BOOL OnInitDialog();
	virtual void OnOK();
	//{{AFX_MSG(CFrameDlg)
		// No message handlers
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

void CFrameDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CFrameDlg)
	//}}AFX_DATA_MAP
}

BOOL CFrameDlg::OnInitDialog()
{
  char buffer[128];
  sprintf(buffer, "%d", ((CDibLookApp *)AfxGetApp())->m_width);
  SetDlgItemText(IDC_EDIT1, buffer);
  sprintf(buffer, "%d", ((CDibLookApp *)AfxGetApp())->m_height);
  SetDlgItemText(IDC_EDIT2, buffer);
  if (((CDibLookApp *)AfxGetApp())->m_xoffset < 0)
	  buffer[0] = 0;
  else
	  sprintf(buffer, "%d", ((CDibLookApp *)AfxGetApp())->m_xoffset);
  SetDlgItemText(IDC_EDIT3, buffer);
  if (((CDibLookApp *)AfxGetApp())->m_yoffset < 0)
	  buffer[0] = 0;
  else
	  sprintf(buffer, "%d", ((CDibLookApp *)AfxGetApp())->m_yoffset);
  SetDlgItemText(IDC_EDIT4, buffer);
  CDialog::OnInitDialog();
  return TRUE;
}

void CFrameDlg::OnOK()
{
  CDialog::OnOK();
  char buffer[128];
  GetDlgItemText(IDC_EDIT1, buffer, sizeof(buffer));
  ((CDibLookApp *)AfxGetApp())->m_width = atoi(buffer);
  GetDlgItemText(IDC_EDIT2, buffer, sizeof(buffer));
  ((CDibLookApp *)AfxGetApp())->m_height = atoi(buffer);
  GetDlgItemText(IDC_EDIT3, buffer, sizeof(buffer));
  if (strlen(buffer))
	  ((CDibLookApp *)AfxGetApp())->m_xoffset = atoi(buffer);
  GetDlgItemText(IDC_EDIT4, buffer, sizeof(buffer));
  if (strlen(buffer))
	  ((CDibLookApp *)AfxGetApp())->m_yoffset = atoi(buffer);
}

BEGIN_MESSAGE_MAP(CFrameDlg, CDialog)
	//{{AFX_MSG_MAP(CFrameDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CDibView::OnPopup1DefineFrame()
{
	CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();

	CFrameDlg frameDlg;
	if (frameDlg.DoModal() == IDOK)
	{
		HDIB hDIB = GetDocument()->GetHDIB();
		if (hDIB == NULL) return;
		LPSTR lpDIB = (LPSTR) ::GlobalLock((HGLOBAL) hDIB);
		int cxDIB = (int) ::DIBWidth(lpDIB);         // Size of DIB - x
		int cyDIB = (int) ::DIBHeight(lpDIB);        // Size of DIB - y
		::GlobalUnlock((HGLOBAL) hDIB);

		if (myapp->m_xoffset >= 0 && myapp->m_xoffset < cxDIB)
		{
			myapp->m_xoffset -= myapp->m_xoffset % GetDocument()->m_MCUwidth;
			m_left = myapp->m_xoffset;
		}
		if (myapp->m_yoffset >= 0 && myapp->m_yoffset < cyDIB)
		{
			myapp->m_yoffset -= myapp->m_yoffset % GetDocument()->m_MCUheight;
			m_top = myapp->m_yoffset;
		}
		if (myapp->m_width <= 0) myapp->m_width = 1;
		if (myapp->m_width > cxDIB) myapp->m_width = cxDIB;
		m_right = cxDIB - m_left - myapp->m_width;
		if (m_right < 0)
		{
			m_left += m_right; m_right = m_left % GetDocument()->m_MCUwidth;
			m_left -= m_right;
		}
		if (myapp->m_height <= 0) myapp->m_height = 1;
		if (myapp->m_height > cyDIB) myapp->m_height = cyDIB;
		m_bottom = cyDIB - m_top - myapp->m_height;
		if (m_bottom < 0)
		{
			m_top += m_bottom; m_bottom = m_top % GetDocument()->m_MCUheight;
			m_top -= m_bottom;
		}

		char buf[64];
		unsigned int paru[2];
		int pari[2];
		sprintf(buf, " %dx%d+%d+%d",
			cxDIB - m_right - m_left,
			cyDIB - m_bottom - m_top, m_left, m_top);
		((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
		((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
		((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);

		GetDocument()->UpdateAllViews(NULL);
	}
}


void CDibView::OnPopup1SaveFrameAs()
{
  GetDocument()->DoSave(NULL, FALSE);
}
